"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var createStore = function (actions, isEnableLog) {
    var currentState = {};
    var currentProvider = null;
    return {
        setProvider: function (provider) {
            if (currentProvider) {
                console.error("Provider is dupplicated!");
                return;
            }
            currentProvider = provider;
            currentState = provider.state;
        },
        getState: function () {
            if (!currentProvider) {
                console.error("Provider is required!");
                return;
            }
            return currentProvider.state;
        },
        dispatch: function (actionType, params) {
            if (!currentProvider) {
                console.error("Provider is required!");
                return;
            }
            // check if action type is exists
            if (!actionType) {
                if (isEnableLog) {
                    (0, utils_1.printWarning)("Action Type is required!");
                }
                return;
            }
            // check if action type is exists
            if (!actions[actionType]) {
                if (isEnableLog) {
                    (0, utils_1.printWarning)("Action with type ".concat(actionType, " is not defined!"));
                }
                return;
            }
            // print prev state
            if (isEnableLog) {
                (0, utils_1.printPrevState)(actionType, currentState);
            }
            // update state changes
            currentState = __assign(__assign({}, currentState), actions[actionType](currentState, params));
            currentProvider.setState(currentState);
            // print next state
            if (isEnableLog) {
                (0, utils_1.printNextState)(currentState);
            }
        },
    };
};
exports.default = createStore;
